@@ -2,10 +2,7 @@
 import re
 import logging
 
-try:
-    from paste.wsgiwrappers import WSGIRequest
-except:
-    pass
+from webob import Request
 
 from routes.base import request_config
 from routes.util import URLGenerator, url_for
@@ -16,7 +13,7 @@ class RoutesMiddleware(object):
     """Routing middleware that handles resolving the PATH_INFO in
     addition to optionally recognizing method overriding."""
     def __init__(self, wsgi_app, mapper, use_method_override=True, 
-                 path_info=True):
+                 path_info=True, singleton=True):
         """Create a Route middleware object
         
         Using the use_method_override keyword will require Paste to be
@@ -35,6 +32,7 @@ class RoutesMiddleware(object):
         """
         self.app = wsgi_app
         self.mapper = mapper
+        self.singleton = singleton
         self.use_method_override = use_method_override
         self.path_info = path_info
         log_debug = self.log_debug = logging.DEBUG >= log.getEffectiveLevel()
@@ -45,43 +43,60 @@ class RoutesMiddleware(object):
     def __call__(self, environ, start_response):
         """Resolves the URL in PATH_INFO, and uses wsgi.routing_args
         to pass on URL resolver results."""
-        config = request_config()
-        config.mapper = self.mapper
-        
         old_method = None
         if self.use_method_override:
-            req = WSGIRequest(environ)
-            req.errors = 'ignore'
-            if '_method' in environ.get('QUERY_STRING', '') and \
-                '_method' in req.GET:
-                old_method = environ['REQUEST_METHOD']
-                environ['REQUEST_METHOD'] = req.GET['_method'].upper()
-                if self.log_debug:
-                    log.debug("_method found in QUERY_STRING, altering request"
-                            " method to %s", environ['REQUEST_METHOD'])
-            elif is_form_post(environ) and '_method' in req.POST:
-                old_method = environ['REQUEST_METHOD']
-                environ['REQUEST_METHOD'] = req.POST['_method'].upper()
-                if self.log_debug:
-                    log.debug("_method found in POST data, altering request "
-                              "method to %s", environ['REQUEST_METHOD'])
+            req = None
+            
+            # In some odd cases, there's no query string
+            try:
+                qs = environ['QUERY_STRING']
+            except KeyError:
+                qs = ''
+            if '_method' in qs:
+                req = Request(environ)
+                req.errors = 'ignore'
+                if '_method' in req.GET:
+                    old_method = environ['REQUEST_METHOD']
+                    environ['REQUEST_METHOD'] = req.GET['_method'].upper()
+                    if self.log_debug:
+                        log.debug("_method found in QUERY_STRING, altering request"
+                                " method to %s", environ['REQUEST_METHOD'])
+            elif environ['REQUEST_METHOD'] == 'POST' and is_form_post(environ):
+                if req is None:
+                    req = Request(environ)
+                    req.errors = 'ignore'
+                if '_method' in req.POST:
+                    old_method = environ['REQUEST_METHOD']
+                    environ['REQUEST_METHOD'] = req.POST['_method'].upper()
+                    if self.log_debug:
+                        log.debug("_method found in POST data, altering request "
+                                  "method to %s", environ['REQUEST_METHOD'])
         
         # Run the actual route matching
         # -- Assignment of environ to config triggers route matching
-        config.environ = environ
-        
-        match = config.mapper_dict
-        route = config.route
+        if self.singleton:
+            config = request_config()
+            config.mapper = self.mapper
+            config.environ = environ
+            match = config.mapper_dict
+            route = config.route
+        else:
+            results = self.mapper.routematch(environ=environ)
+            if results:
+                match, route = results[0], results[1]
+            else:
+                match = route = None
                 
         if old_method:
             environ['REQUEST_METHOD'] = old_method
         
-        urlinfo = "%s %s" % (environ['REQUEST_METHOD'], environ['PATH_INFO'])
         if not match:
             match = {}
             if self.log_debug:
+                urlinfo = "%s %s" % (environ['REQUEST_METHOD'], environ['PATH_INFO'])
                 log.debug("No route matched for %s", urlinfo)
         elif self.log_debug:
+            urlinfo = "%s %s" % (environ['REQUEST_METHOD'], environ['PATH_INFO'])
             log.debug("Matched %s", urlinfo)
             log.debug("Route path: '%s', defaults: %s", route.routepath, 
                       route.defaults)
@@ -92,7 +107,7 @@ class RoutesMiddleware(object):
         environ['routes.route'] = route
         environ['routes.url'] = url
 
-        if hasattr(route, 'redirect'):
+        if route and route.redirect:
             route_name = '_redirect_%s' % id(route)
             location = url_for(route_name, **match)
             log.debug("Using redirect route, redirect to '%s' with status"
@@ -104,22 +119,19 @@ class RoutesMiddleware(object):
 
         # If the route included a path_info attribute and it should be used to
         # alter the environ, we'll pull it out
-        if self.path_info and match.get('path_info'):
+        if self.path_info and 'path_info' in match:
             oldpath = environ['PATH_INFO']
             newpath = match.get('path_info') or ''
             environ['PATH_INFO'] = newpath
             if not environ['PATH_INFO'].startswith('/'):
                 environ['PATH_INFO'] = '/' + environ['PATH_INFO']
-            environ['SCRIPT_NAME'] += re.sub(r'^(.*?)/' + newpath + '$', 
+            environ['SCRIPT_NAME'] += re.sub(r'^(.*?)/' + re.escape(newpath) + '$', 
                                              r'\1', oldpath)
-            if environ['SCRIPT_NAME'].endswith('/'):
-                environ['SCRIPT_NAME'] = environ['SCRIPT_NAME'][:-1]
         
         response = self.app(environ, start_response)
         
         # Wrapped in try as in rare cases the attribute will be gone already
         try:
-            del config.environ
             del self.mapper.environ
         except AttributeError:
             pass
@@ -127,8 +139,6 @@ class RoutesMiddleware(object):
 
 def is_form_post(environ):
     """Determine whether the request is a POSTed html form"""
-    if environ['REQUEST_METHOD'] != 'POST':
-        return False
     content_type = environ.get('CONTENT_TYPE', '').lower()
     if ';' in content_type:
         content_type = content_type.split(';', 1)[0]
