@@ -11,7 +11,19 @@ import urllib
 from routes import request_config
 
 
-def _screenargs(kargs, mapper, environ):
+class RoutesException(Exception):
+    """Tossed during Route exceptions"""
+
+
+class MatchException(RoutesException):
+    """Tossed during URL matching exceptions"""
+
+
+class GenerationException(RoutesException):
+    """Tossed during URL generation exceptions"""
+
+
+def _screenargs(kargs, mapper, environ, force_explicit=False):
     """
     Private function that takes a dict, and screens it against the current 
     request dict to determine what the dict should look like that is used. 
@@ -23,9 +35,9 @@ def _screenargs(kargs, mapper, environ):
         if isinstance(val, unicode):
             kargs[key] = val.encode(encoding)
     
-    if mapper.explicit and mapper.sub_domains:
+    if mapper.explicit and mapper.sub_domains and not force_explicit:
         return _subdomain_check(kargs, mapper, environ)
-    elif mapper.explicit:
+    elif mapper.explicit and not force_explicit:
         return kargs
     
     controller_name = kargs.get('controller')
@@ -67,13 +79,10 @@ def _subdomain_check(kargs, mapper, environ):
         if isinstance(subdomain, unicode):
             subdomain = str(subdomain)
         
-        # We use a try/except here, cause the only time there should be no
-        # environ is when we're unit testing, in which case we shouldn't be
-        # changing kargs and such. The exception catching also won't hurt as
-        # badly here vs doing a hasattr on every url check
-        try:
-            fullhost = environ.get('HTTP_HOST') or environ.get('SERVER_NAME')
-        except AttributeError:
+        fullhost = environ.get('HTTP_HOST') or environ.get('SERVER_NAME')
+        
+        # In case environ defaulted to {}
+        if not fullhost:
             return kargs
         
         hostmatch = fullhost.split(':')
@@ -118,8 +127,6 @@ def _str_encode(string, encoding):
             s = string
         else:
             s = unicode(string).encode(encoding)
-    else:
-        s = str(string)
     return s
 
 
@@ -176,8 +183,6 @@ def url_for(*args, **kargs):
     for key in ['anchor', 'host', 'protocol']:
         if kargs.get(key):
             del kargs[key]
-        if kargs.has_key(key+'_'):
-            kargs[key] = kargs.pop(key+'_')
     config = request_config()
     route = None
     static = False
@@ -320,8 +325,6 @@ class URLGenerator(object):
         for key in ['anchor', 'host', 'protocol']:
             if kargs.get(key):
                 del kargs[key]
-            if kargs.has_key(key+'_'):
-                kargs[key] = kargs.pop(key+'_')
         
         route = None
         use_current = '_use_current' in kargs and kargs.pop('_use_current')
@@ -329,7 +332,7 @@ class URLGenerator(object):
         static = False
         encoding = self.mapper.encoding
         url = ''
-        
+                
         more_args = len(args) > 0
         if more_args:
             route = self.mapper._routenames.get(args[0])
@@ -337,7 +340,6 @@ class URLGenerator(object):
         if not route and more_args:
             static = True
             url = args[0]
-
             if url.startswith('/') and self.environ.get('SCRIPT_NAME'):
                 url = self.environ.get('SCRIPT_NAME') + url
 
@@ -367,20 +369,19 @@ class URLGenerator(object):
                 # If this route has a filter, apply it
                 if route.filter:
                     newargs = route.filter(newargs)
-                
                 if not route.static or (route.static and not route.external):
                     # Handle sub-domains
                     newargs = _subdomain_check(newargs, self.mapper,
                                                self.environ)
             elif use_current:
-                newargs = _screenargs(kargs, self.mapper, self.environ)
+                newargs = _screenargs(kargs, self.mapper, self.environ, force_explicit=True)
             elif 'sub_domain' in kargs:
                 newargs = _subdomain_check(kargs, self.mapper, self.environ)
             else:
                 newargs = kargs
-            anchor = newargs.pop('_anchor', None) or anchor
-            host = newargs.pop('_host', None) or host
-            protocol = newargs.pop('_protocol', None) or protocol
+            anchor = anchor or newargs.pop('_anchor', None)
+            host = host or newargs.pop('_host', None)
+            protocol = protocol or newargs.pop('_protocol', None)
             url = self.mapper.generate(*route_args, **newargs)
         if anchor is not None:
             url += '#' + _url_quote(anchor, encoding)
@@ -398,7 +399,9 @@ class URLGenerator(object):
             if not protocol:
                 protocol = hostinfo['protocol']
             if url is not None:
-                url = protocol + '://' + host + url
+                if host[-1] != '/':
+                    host += '/'
+                url = protocol + '://' + host + url.lstrip('/')
 
         if not isinstance(url, str) and url is not None:
             raise GenerationException("Can only return a string, got "
@@ -491,15 +494,3 @@ def controller_scan(directory=None):
     controllers = find_controllers(directory)
     controllers.sort(longest_first)
     return controllers
-
-
-class RoutesException(Exception):
-    """Tossed during Route exceptions"""
-
-
-class MatchException(RoutesException):
-    """Tossed during URL matching exceptions"""
-
-
-class GenerationException(RoutesException):
-    """Tossed during URL generation exceptions"""
